import numpy as np
import datetime
import requests as req

def BMM_mean(x):
  return np.mean(x, axis=0)

def BMM_min(x):
  return np.min(x, axis=0)

BASE_METRIC_MODIFIERS = {
  'avg': BMM_mean,
  'min': BMM_min
}

class RunLog:
  def __init__(self, _name, _modifiers=BASE_METRIC_MODIFIERS, _log=None):
    self.fullname = _name + datetime.datetime.now().strftime("%y%m%d_%H%M")
    self.modifiers = _modifiers
    self.log = _log
    self.config = {}
    self.data = []
    self.currentEpoch = {}

  def __getitem__(self, _i):
    return self.config[_i]

  def __setitem__(self, _i, _v):
    self.config[_i] = _v

  def save(self):
    if self.log is not None:
      req.delete(self.log + self.fullname)

  def __call__(self, _metric, _value):
    t = _metric.split(':')
    metric = t[0]
    modifiers = t[1:]
    if metric not in self.currentEpoch:
      self.currentEpoch[metric] = [_value, set()]
    else:
      if not isinstance(self.currentEpoch[metric][0], list):
        self.currentEpoch[metric][0] = [self.currentEpoch[metric][0], _value]
      else:
        self.currentEpoch[metric][0].append(_value)
    if len(modifiers):
      self.currentEpoch[metric][1].update(modifiers)

  def step(self):
    # Lazy starting
    if len(self.data) == 0 and self.log is not None:
      req.post(self.log + self.fullname, json=self.config)
      
    epoch = {}
    for key in self.currentEpoch:
      metric, modifiers = self.currentEpoch[key]

      for modifier in modifiers:
        metric = self.modifiers[modifier](metric)
        epoch[key + ':' + modifier] = metric.item() if metric.size == 1 else metric.tolist()
      if len(modifiers) == 0:
        epoch[key] = metric.item() if metric.size == 1 else metric.tolist()

    self.data.append(epoch)
    if self.log is not None:
      req.put(self.log + self.fullname, json=epoch)

    self.currentEpoch = {}
